"use strict";
/**
 * @fileoverview Story files should have a default export
 * @author Yann Braga
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const path_1 = __importDefault(require("path"));
const constants_1 = require("../utils/constants");
const ast_1 = require("../utils/ast");
const create_storybook_rule_1 = require("../utils/create-storybook-rule");
module.exports = (0, create_storybook_rule_1.createStorybookRule)({
    name: 'default-exports',
    defaultOptions: [],
    meta: {
        type: 'problem',
        severity: 'error',
        docs: {
            description: 'Story files should have a default export',
            categories: [constants_1.CategoryId.CSF, constants_1.CategoryId.RECOMMENDED],
        },
        messages: {
            shouldHaveDefaultExport: 'The file should have a default export.',
            fixSuggestion: 'Add default export',
        },
        fixable: 'code',
        hasSuggestions: true,
        schema: [],
    },
    create(context) {
        //----------------------------------------------------------------------
        // Helpers
        //----------------------------------------------------------------------
        const getComponentName = (node, filePath) => {
            const name = path_1.default.basename(filePath).split('.')[0];
            const imported = node.body.find((stmt) => {
                if ((0, ast_1.isImportDeclaration)(stmt) &&
                    (0, ast_1.isLiteral)(stmt.source) &&
                    stmt.source.value.startsWith(`./${name}`)) {
                    return !!stmt.specifiers.find((spec) => (0, ast_1.isIdentifier)(spec.local) && spec.local.name === name);
                }
            });
            return imported ? name : null;
        };
        //----------------------------------------------------------------------
        // Public
        //----------------------------------------------------------------------
        let hasDefaultExport = false;
        let localMetaNode;
        let hasStoriesOfImport = false;
        return {
            ImportSpecifier(node) {
                if (node.imported.name === 'storiesOf') {
                    hasStoriesOfImport = true;
                }
            },
            VariableDeclaration(node) {
                var _a, _b;
                // Take `const meta = {};` into consideration
                if (node.kind === 'const' &&
                    node.declarations.length === 1 &&
                    ((_a = node.declarations[0]) === null || _a === void 0 ? void 0 : _a.id.type) === 'Identifier' &&
                    ((_b = node.declarations[0]) === null || _b === void 0 ? void 0 : _b.id.name) === 'meta') {
                    localMetaNode = node;
                }
            },
            ExportDefaultSpecifier: function () {
                hasDefaultExport = true;
            },
            ExportDefaultDeclaration: function () {
                hasDefaultExport = true;
            },
            'Program:exit': function (program) {
                var _a;
                if (!hasDefaultExport && !hasStoriesOfImport) {
                    const componentName = getComponentName(program, context.filename);
                    const firstNonImportStatement = program.body.find((n) => !(0, ast_1.isImportDeclaration)(n));
                    const node = (_a = firstNonImportStatement !== null && firstNonImportStatement !== void 0 ? firstNonImportStatement : program.body[0]) !== null && _a !== void 0 ? _a : program;
                    const report = {
                        node,
                        messageId: 'shouldHaveDefaultExport',
                    };
                    const fix = (fixer) => {
                        const sourceCode = context.sourceCode.getText();
                        // only add semicolons if needed
                        const semiCharacter = sourceCode.includes(';') ? ';' : '';
                        if (localMetaNode) {
                            const exportStatement = `\nexport default meta${semiCharacter}`;
                            return fixer.insertTextAfter(localMetaNode, exportStatement);
                        }
                        else {
                            const exportStatement = componentName
                                ? `export default { component: ${componentName} }${semiCharacter}\n`
                                : `export default {}${semiCharacter}\n`;
                            return fixer.insertTextBefore(node, exportStatement);
                        }
                    };
                    context.report(Object.assign(Object.assign({}, report), { fix, suggest: [
                            {
                                messageId: 'fixSuggestion',
                                fix,
                            },
                        ] }));
                }
            },
        };
    },
});
